
#ifdef WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "aes.h"
#include "pwd2key.h"
#include "aesopt.h"

#include "kryptwit.h"

void randombytes(unsigned char *buf, unsigned int len)
{
#ifdef WIN32
	HCRYPTPROV hCryptProv;
	if(CryptAcquireContext(&hCryptProv,NULL,NULL,PROV_RSA_FULL,0) == 0)
  	CryptAcquireContext(&hCryptProv,NULL,NULL,PROV_RSA_FULL,CRYPT_NEWKEYSET);  	  
  CryptGenRandom(hCryptProv, len, buf);
  CryptReleaseContext(hCryptProv, 0);  
#elif unix
	FILE *fp=NULL;
	fp = fopen("/dev/urandom", "rb");
	if(!fp)
		return;
	fread(buf, 1, len, fp);
	fclose(fp);
#endif
}

void rfc3686_inc(unsigned char ctr_buf[AES_BLOCK_SIZE])
{
	if(!(++(ctr_buf[15])))
		if(!(++(ctr_buf[14])))
			if(!(++(ctr_buf[13])))
				++(ctr_buf[12]);
}


static const char lookup[] = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!`#$%&/()='?_{+*~}-.,[]@";

// assume len mod 4 = 0
static void ascii85_encode(unsigned char *in, char *out, const unsigned int len)
{
	unsigned int i, t;
	for(i=0; i < len; i += 4)
	{
		t = word_in(in, i/4);
		*out++ = lookup[t % 85]; t /= 85;
		*out++ = lookup[t % 85]; t /= 85;
		*out++ = lookup[t % 85]; t /= 85;
		*out++ = lookup[t % 85]; t /= 85;
		*out++ = lookup[t % 85]; t /= 85;
	}
	*out++ = 0;
}

static int get_pos(char c)
{
	int i;
	for(i=0; i < 85; ++i)
	  if(c == lookup[i])
	  	return i;
	return -1;
}
// len mod 5 == 0
static void ascii85_decode(char *in, unsigned char *out, const unsigned int len)
{
	unsigned int i;
	for(i=0; i < len; i += 5)
	{
		unsigned int t=0;
		t += get_pos(in[i+4]); t*=85;
		t += get_pos(in[i+3]); t*=85;
		t += get_pos(in[i+2]); t*=85;
		t += get_pos(in[i+1]); t*=85;
		t += get_pos(in[i+0]); //t*=85;
		word_out(out, i/5, t);
	}
}

// msgout must have >= 140 chars, msgin can't exceed 100
int status_update_encrypt(char *msgout, char *msgin, const char *username, const char *password)
{
	unsigned char key[32]; // Holds a 256-bit key
	aes_encrypt_ctx aeskey[1];
	unsigned char ctr[16]; // block size
	unsigned char ct[104];
	unsigned long len;
	
	if( !msgout || !msgin || !username || !password )
		return NULL_POINTER; // At least one of them is NULL
		
	if(strlen(username) > 20)
		return USERNAME_TOO_LARGE;
		
	len = strlen(msgin);
		
	if( len > 100 )
		return MSG_TOO_LARGE; // Too large
		
	memset(ctr, 0, 16);
	memset(ct, 0, 104);
	memcpy(ct, msgin, len);
	memset(key, 0, 32);
	derive_key((unsigned char*)password, strlen(password), (unsigned char*)username, strlen(username), 1000, key, 32);
	aes_encrypt_key(key, 256, aeskey);
	
	randombytes(ctr, 8); // Random IV
	aes_ctr_encrypt(ct, ct, 100, ctr, rfc3686_inc, aeskey);
	
	memset(msgout, lookup[0], 4+10+125);
	memcpy(msgout, ";-) ", 4);
	ascii85_encode(ctr, msgout+4, 8); // IV
	ascii85_encode(ct, msgout+14, 100); // Message
	msgout[4+10+125] = 0;
  return 0;	
}


int status_update_decrypt(char *msgout, char *msgin, const char *username, const char *password)
{
	unsigned char key[32]; // Holds a 256-bit key
	aes_encrypt_ctx aeskey[1];
	unsigned char ctr[16]; // block size
	unsigned char ct[104];
	unsigned long len;
	
	if( !msgout || !msgin || !username || !password )
		return NULL_POINTER; // At least one of them is NULL
		
	if(strlen(username) > 20)
		return USERNAME_TOO_LARGE;
		
	len = strlen(msgin);
		
	if( len != 139 )
		return MSG_WRONG_LENGTH; // Invalid length
		
	if(memcmp(msgin, ";-) ", 4))
		return MSG_WRONG_PREFIX;
	
	memset(ctr, 0, 16);
	ascii85_decode(msgin+4, ctr, 10);
	
	memset(ct, 0, 104);
	ascii85_decode(msgin+14, ct, 125);
	memset(key, 0, 32);
	derive_key((unsigned char*)password, strlen(password), (unsigned char*)username, strlen(username), 1000, key, 32);
	aes_encrypt_key(key, 256, aeskey);
	aes_ctr_encrypt(ct, (unsigned char*)msgout, 100, ctr, rfc3686_inc, aeskey);    
  return 0;	
}

